@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -151,6 +151,8 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
          *
          * This function can be safely used for constructing (and later
          * destructing) objects even without any OpenGL context being active.
+         * However note that this is a low-level and a potentially dangerous
+         * API, see the documentation of @ref NoCreate for alternatives.
          * @see @ref CubeMapTextureArray(), @ref wrap()
          */
         explicit CubeMapTextureArray(NoCreateT) noexcept: AbstractTexture{NoCreate, GL_TEXTURE_CUBE_MAP_ARRAY} {}
@@ -379,7 +381,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
          * See @ref Texture::setSrgbDecode() for more information.
          * @requires_extension Extension @gl_extension{EXT,texture_sRGB_decode}
          * @requires_es_extension Extension @gl_extension{ANDROID,extension_pack_es31a} /
-         *      @gl_extension2{EXT,texture_sRGB_decode,texture_sRGB_decode}
+         *      @gl_extension{EXT,texture_sRGB_decode}
          */
         CubeMapTextureArray& setSrgbDecode(bool decode) {
             AbstractTexture::setSrgbDecode(decode);
@@ -388,7 +390,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
 
         #ifdef MAGNUM_BUILD_DEPRECATED
         /** @brief @copybrief setSrgbDecode()
-         * @deprecated Use @ref setSrgbDecode() instead.
+         * @m_deprecated_since{2018,10} Use @ref setSrgbDecode() instead.
          */
         CORRADE_DEPRECATED("use setSrgbDecode() instead") CubeMapTextureArray& setSRGBDecode(bool decode) {
             return setSrgbDecode(decode);
@@ -466,7 +468,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
 
         #ifndef MAGNUM_TARGET_GLES
         /**
-         * @brief @copybrief Texture::image(Int, Image&)
+         * @brief Read given texture mip level to an image
          *
          * See @ref Texture::image(Int, Image&) for more information.
          * @requires_gl Texture image queries are not available in OpenGL ES.
@@ -486,7 +488,19 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         Image3D image(Int level, Image3D&& image);
 
         /**
-         * @brief @copybrief Texture::image(Int, BufferImage&, BufferUsage)
+         * @brief Read given texture mip level to an image view
+         * @m_since{2019,10}
+         *
+         * Compared to @ref image(Int, Image3D&) the function reads the pixels
+         * into the memory provided by @p image, expecting it's not
+         * @cpp nullptr @ce and its size is the same as size of given @p level.
+         */
+        void image(Int level, const MutableImageView3D& image) {
+            AbstractTexture::image<3>(level, image);
+        }
+
+        /**
+         * @brief Read given texture mip level to a buffer image
          *
          * See @ref Texture::image(Int, BufferImage&, BufferUsage) for more
          * information.
@@ -507,7 +521,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         BufferImage3D image(Int level, BufferImage3D&& image, BufferUsage usage);
 
         /**
-         * @brief @copybrief Texture::compressedImage(Int, CompressedImage&)
+         * @brief Read given compressed texture mip level to an image
          *
          * See @ref Texture::compressedImage(Int, CompressedImage&) for more
          *      information.
@@ -530,7 +544,20 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         CompressedImage3D compressedImage(Int level, CompressedImage3D&& image);
 
         /**
-         * @brief @copybrief Texture::compressedImage(Int, CompressedBufferImage&, BufferUsage)
+         * @brief Read given compressed texture mip level to an image view
+         * @m_since{2019,10}
+         *
+         * Compared to @ref compressedImage(Int, CompressedImage3D&) the
+         * function reads the pixels into the memory provided by @p image,
+         * expecting it's not @cpp nullptr @ce, its format is the same as
+         * texture format and its size is the same as size of given @p level.
+         */
+        void compressedImage(Int level, const MutableCompressedImageView3D& image) {
+            AbstractTexture::compressedImage<3>(level, image);
+        }
+
+        /**
+         * @brief Read given compressed texture mip level to a buffer image
          *
          * See @ref Texture::compressedImage(Int, CompressedBufferImage&, BufferUsage)
          * for more information.
@@ -553,7 +580,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         CompressedBufferImage3D compressedImage(Int level, CompressedBufferImage3D&& image, BufferUsage usage);
 
         /**
-         * @brief @copybrief Texture::subImage(Int, const RangeTypeFor<dimensions, Int>&, Image&)
+         * @brief Read a range of given texture mip level to an image
          *
          * See @ref Texture::subImage(Int, const RangeTypeFor<dimensions, Int>&, Image&)
          * for more information.
@@ -575,7 +602,20 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         Image3D subImage(Int level, const Range3Di& range, Image3D&& image);
 
         /**
-         * @brief @copybrief Texture::subImage(Int, const RangeTypeFor<dimensions, Int>&, BufferImage&, BufferUsage)
+         * @brief Read a range of given texture mip level to an image view
+         * @m_since{2019,10}
+         *
+         * Compared to @ref subImage(Int, const Range3Di&, Image3D&) the
+         * function reads the pixels into the memory provided by @p image,
+         * expecting it's not @cpp nullptr @ce and its size is the same as
+         * @p range size.
+         */
+        void subImage(Int level, const Range3Di& range, const MutableImageView3D& image) {
+            AbstractTexture::subImage<3>(level, range, image);
+        }
+
+        /**
+         * @brief Read a range of given texture mip level to a buffer image
          *
          * See @ref Texture::subImage(Int, const RangeTypeFor<dimensions, Int>&, BufferImage&, BufferUsage)
          * for more information.
@@ -597,7 +637,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         BufferImage3D subImage(Int level, const Range3Di& range, BufferImage3D&& image, BufferUsage usage);
 
         /**
-         * @brief @copybrief Texture::compressedSubImage(Int, const RangeTypeFor<dimensions, Int>&, CompressedImage&)
+         * @brief Read a range of given compressed texture mip level to an image
          *
          * See @ref Texture::compressedSubImage(Int, const RangeTypeFor<dimensions, Int>&, CompressedImage&)
          * for more information.
@@ -625,7 +665,20 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         CompressedImage3D compressedSubImage(Int level, const Range3Di& range, CompressedImage3D&& image);
 
         /**
-         * @brief @copybrief Texture::compressedSubImage(Int, const RangeTypeFor<dimensions, Int>&, CompressedBufferImage&, BufferUsage)
+         * @brief Read a range of given compressed texture mip level to an image view
+         * @m_since{2019,10}
+         *
+         * Compared to @ref compressedSubImage(Int, const Range3Di&, CompressedImage3D&)
+         * the function reads the pixels into the memory provided by @p image,
+         * expecting it's not @cpp nullptr @ce, its format is the same as
+         * texture format and its size is the same as @p range size.
+         */
+        void compressedSubImage(Int level, const Range3Di& range, const MutableCompressedImageView3D& image) {
+            AbstractTexture::compressedSubImage<3>(level, range, image);
+        }
+
+        /**
+         * @brief Read a range of given compressed texture mip level to a buffer image
          *
          * See @ref Texture::compressedSubImage(Int, const RangeTypeFor<dimensions, Int>&, CompressedBufferImage&, BufferUsage)
          * for more information.
@@ -843,18 +896,7 @@ class MAGNUM_GL_EXPORT CubeMapTextureArray: public AbstractTexture {
         explicit CubeMapTextureArray(GLuint id, ObjectFlags flags) noexcept: AbstractTexture{id, GL_TEXTURE_CUBE_MAP_ARRAY, flags} {}
 };
 
-}
-
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* Note: needs to be prefixed with Magnum:: otherwise Doxygen can't find it */
-
-/** @brief @copybrief GL::CubeMapTextureArray
- * @deprecated Use @ref GL::CubeMapTextureArray instead.
- */
-typedef CORRADE_DEPRECATED("use GL::CubeMapTextureArray instead") Magnum::GL::CubeMapTextureArray CubeMapTextureArray;
-#endif
-
-}
+}}
 #else
 #error this header is not available in OpenGL ES 2.0 and WebGL build
 #endif
