@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -55,22 +55,22 @@ Example usage:
     @ref Quaternion::transformVectorNormalized()
 @todo GPU transform feedback implementation (otherwise this is only bad joke)
 */
-template<class T, class U> void transformVectorsInPlace(const Math::Matrix4<T>& matrix, U& vectors) {
+template<class T, class U> void transformVectorsInPlace(const Math::Matrix4<T>& matrix, U&& vectors) {
     for(auto& vector: vectors) vector = matrix.transformVector(vector);
 }
 
 /** @overload */
-template<class T, class U> void transformVectorsInPlace(const Math::Matrix3<T>& matrix, U& vectors) {
+template<class T, class U> void transformVectorsInPlace(const Math::Matrix3<T>& matrix, U&& vectors) {
     for(auto& vector: vectors) vector = matrix.transformVector(vector);
 }
 
 /** @overload */
-template<class T, class U> void transformVectorsInPlace(const Math::Complex<T>& complex, U& vectors) {
+template<class T, class U> void transformVectorsInPlace(const Math::Complex<T>& complex, U&& vectors) {
     for(auto& vector: vectors) vector = complex.transformVector(vector);
 }
 
 /** @overload */
-template<class T, class U> void transformVectorsInPlace(const Math::Quaternion<T>& normalizedQuaternion, U& vectors) {
+template<class T, class U> void transformVectorsInPlace(const Math::Quaternion<T>& normalizedQuaternion, U&& vectors) {
     for(auto& vector: vectors) vector = normalizedQuaternion.transformVectorNormalized(vector);
 }
 
@@ -106,22 +106,22 @@ Example usage:
     @ref Matrix4::transformPoint(),
     @ref DualQuaternion::transformPointNormalized()
 */
-template<class T, class U> void transformPointsInPlace(const Math::Matrix4<T>& matrix, U& points) {
+template<class T, class U> void transformPointsInPlace(const Math::Matrix4<T>& matrix, U&& points) {
     for(auto& point: points) point = matrix.transformPoint(point);
 }
 
 /** @overload */
-template<class T, class U> void transformPointsInPlace(const Math::Matrix3<T>& matrix, U& points) {
+template<class T, class U> void transformPointsInPlace(const Math::Matrix3<T>& matrix, U&& points) {
     for(auto& point: points) point = matrix.transformPoint(point);
 }
 
 /** @overload */
-template<class T, class U> void transformPointsInPlace(const Math::DualComplex<T>& dualComplex, U& points) {
+template<class T, class U> void transformPointsInPlace(const Math::DualComplex<T>& dualComplex, U&& points) {
     for(auto& point: points) point = dualComplex.transformPoint(point);
 }
 
 /** @overload */
-template<class T, class U> void transformPointsInPlace(const Math::DualQuaternion<T>& normalizedDualQuaternion, U& points) {
+template<class T, class U> void transformPointsInPlace(const Math::DualQuaternion<T>& normalizedDualQuaternion, U&& points) {
     for(auto& point: points) point = normalizedDualQuaternion.transformPointNormalized(point);
 }
 
