@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -87,7 +87,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Transform object
+         * @brief Transform the object
          * @return Reference to self (for method chaining)
          *
          * Expects that the matrix represents rigid transformation.
@@ -101,7 +101,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Transform object as a local transformation
+         * @brief Transform the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others.
@@ -113,18 +113,8 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(transformation);
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
         /**
-         * @copybrief transform()
-         * @deprecated Use @ref transform() or @ref transformLocal() instead.
-         */
-        CORRADE_DEPRECATED("use transform() or transformLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& transform(const Math::Matrix4<T>& transformation, TransformationType type) {
-            return type == TransformationType::Global ? transform(transformation) : transformLocal(transformation);
-        }
-        #endif
-
-        /**
-         * @brief Translate object
+         * @brief Translate the object
          * @return Reference to self (for method chaining)
          *
          * Same as calling @ref transform() with @ref Math::Matrix4::translation().
@@ -136,7 +126,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Translate object as a local transformation
+         * @brief Translate the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -146,34 +136,46 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(Math::Matrix4<T>::translation(vector));
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
         /**
-         * @copybrief translate()
-         * @deprecated Use @ref translate() or @ref translateLocal() instead.
+         * @brief Rotate the object using a quaternion
+         * @param quaternion    Normalized quaternion
+         * @return Reference to self (for method chaining)
+         * @m_since{2020,06}
+         *
+         * Expects that the quaternion is normalized.
+         * @see @ref rotate(Math::Rad<T>, const Math::Vector3<T>&),
+         *      @ref rotateLocal(const Math::Quaternion<T>&), @ref rotateX(),
+         *      @ref rotateY(), @ref rotateZ()
          */
-        CORRADE_DEPRECATED("use translate() or translateLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& translate(const Math::Vector3<T>& vector, TransformationType type) {
-            return type == TransformationType::Global ? translate(vector) : translateLocal(vector);
-        }
-        #endif
+        Object<BasicRigidMatrixTransformation3D<T>>& rotate(const Math::Quaternion<T>& quaternion);
 
         /**
-         * @brief Rotate object
+         * @brief Rotate the object using a quaternion as a local transformation
+         * @m_since{2020,06}
+         *
+         * Similar to the above, except that the transformation is applied
+         * before all others.
+         */
+        Object<BasicRigidMatrixTransformation3D<T>>& rotateLocal(const Math::Quaternion<T>& quaternion);
+
+        /**
+         * @brief Rotate the object
          * @param angle             Angle (counterclockwise)
          * @param normalizedAxis    Normalized rotation axis
          * @return Reference to self (for method chaining)
          *
          * Same as calling @ref transform() with @ref Math::Matrix4::rotation().
-         * @see @ref rotateLocal(), @ref rotateX(), @ref rotateY(),
-         *      @ref rotateZ(), @ref normalizeRotation(),
-         *      @ref Math::Vector3::xAxis(), @ref Math::Vector3::yAxis(),
-         *      @ref Math::Vector3::zAxis()
+         * @see @ref rotate(const Math::Quaternion<T>&), @ref rotateLocal(),
+         *      @ref rotateX(), @ref rotateY(), @ref rotateZ(),
+         *      @ref normalizeRotation(), @ref Math::Vector3::xAxis(),
+         *      @ref Math::Vector3::yAxis(), @ref Math::Vector3::zAxis()
          */
         Object<BasicRigidMatrixTransformation3D<T>>& rotate(Math::Rad<T> angle, const Math::Vector3<T>& normalizedAxis) {
             return transformInternal(Math::Matrix4<T>::rotation(angle, normalizedAxis));
         }
 
         /**
-         * @brief Rotate object as a local transformation
+         * @brief Rotate the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -183,18 +185,8 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(Math::Matrix4<T>::rotation(angle, normalizedAxis));
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
         /**
-         * @copybrief rotate()
-         * @deprecated Use @ref rotate() or @ref rotateLocal() instead.
-         */
-        CORRADE_DEPRECATED("use rotate() or rotateLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& rotate(Math::Rad<T> angle, const Math::Vector3<T>& normalizedAxis, TransformationType type) {
-            return type == TransformationType::Global ? rotate(angle, normalizedAxis) : rotateLocal(angle, normalizedAxis);
-        }
-        #endif
-
-        /**
-         * @brief Rotate object around X axis
+         * @brief Rotate the object around X axis
          * @param angle             Angle (counterclockwise)
          * @return Reference to self (for method chaining)
          *
@@ -206,7 +198,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Rotate object around X axis as a local transformation
+         * @brief Rotate the object around X axis as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -216,18 +208,8 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(Math::Matrix4<T>::rotationX(angle));
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
         /**
-         * @copybrief rotateX()
-         * @deprecated Use @ref rotateX() or @ref rotateXLocal() instead.
-         */
-        CORRADE_DEPRECATED("use rotateX() or rotateXLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& rotateX(Math::Rad<T> angle, TransformationType type) {
-            return type == TransformationType::Global ? rotateX(angle) : rotateXLocal(angle);
-        }
-        #endif
-
-        /**
-         * @brief Rotate object around Y axis
+         * @brief Rotate the object around Y axis
          * @param angle             Angle (counterclockwise)
          * @return Reference to self (for method chaining)
          *
@@ -239,7 +221,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Rotate object around Y axis as a local transformation
+         * @brief Rotate the object around Y axis as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -249,18 +231,8 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(Math::Matrix4<T>::rotationY(angle));
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
         /**
-         * @copybrief rotateY()
-         * @deprecated Use @ref rotateY() or @ref rotateYLocal() instead.
-         */
-        CORRADE_DEPRECATED("use rotateY() or rotateYLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& rotateY(Math::Rad<T> angle, TransformationType type) {
-            return type == TransformationType::Global ? rotateY(angle) : rotateYLocal(angle);
-        }
-        #endif
-
-        /**
-         * @brief Rotate object around Z axis
+         * @brief Rotate the object around Z axis
          * @param angle             Angle (counterclockwise)
          * @return Reference to self (for method chaining)
          *
@@ -272,7 +244,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Rotate object around Z axis as a local transformation
+         * @brief Rotate the object around Z axis as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -282,18 +254,8 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(Math::Matrix4<T>::rotationZ(angle));
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
         /**
-         * @copybrief rotateZ()
-         * @deprecated Use @ref rotateZ() or @ref rotateZLocal() instead.
-         */
-        CORRADE_DEPRECATED("use rotateZ() or rotateZLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& rotateZ(Math::Rad<T> angle, TransformationType type) {
-            return type == TransformationType::Global ? rotateZ(angle) : rotateZLocal(angle);
-        }
-        #endif
-
-        /**
-         * @brief Reflect object
+         * @brief Reflect the object
          * @param normal    Normal of the plane through which to reflect
          *      (normalized)
          * @return Reference to self (for method chaining)
@@ -306,7 +268,7 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         }
 
         /**
-         * @brief Reflect object as a local transformation
+         * @brief Reflect the object as a local transformation
          *
          * Similar to the above, except that the transformation is applied
          * before all others. Same as calling @ref transformLocal() with
@@ -316,16 +278,6 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
             return transformLocalInternal(Math::Matrix4<T>::reflection(normal));
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @copybrief reflect()
-         * @deprecated Use @ref reflect() or @ref reflectLocal() instead.
-         */
-        CORRADE_DEPRECATED("use reflect() or reflectLocal() instead") Object<BasicRigidMatrixTransformation3D<T>>& reflect(const Math::Vector3<T>& normal, TransformationType type) {
-            return type == TransformationType::Global ? reflect(normal) : reflectLocal(normal);
-        }
-        #endif
-
     protected:
         /* Allow construction only from Object */
         explicit BasicRigidMatrixTransformation3D() = default;
@@ -336,6 +288,13 @@ template<class T> class BasicRigidMatrixTransformation3D: public AbstractBasicTr
         void doTranslate(const Math::Vector3<T>& vector) override final { translate(vector); }
         void doTranslateLocal(const Math::Vector3<T>& vector) override final { translateLocal(vector); }
 
+        void doRotate(const Math::Quaternion<T>& quaternion) override final {
+            rotate(quaternion);
+        }
+        void doRotateLocal(const Math::Quaternion<T>& quaternion) override final {
+            rotateLocal(quaternion);
+        }
+
         void doRotate(Math::Rad<T> angle, const Math::Vector3<T>& normalizedAxis) override final {
             rotate(angle, normalizedAxis);
         }
@@ -407,7 +366,8 @@ template<class T> struct Transformation<BasicRigidMatrixTransformation3D<T>> {
 
 }
 
-#if defined(CORRADE_TARGET_WINDOWS) && !defined(__MINGW32__)
+#if defined(CORRADE_TARGET_WINDOWS) && !(defined(CORRADE_TARGET_MINGW) && !defined(CORRADE_TARGET_CLANG))
+extern template class MAGNUM_SCENEGRAPH_EXPORT BasicRigidMatrixTransformation3D<Float>;
 extern template class MAGNUM_SCENEGRAPH_EXPORT Object<BasicRigidMatrixTransformation3D<Float>>;
 #endif
 
